@@ -54,7 +54,11 @@ void vld_branch_info_free(vld_branch_info *branch_info)
 void vld_branch_info_update(vld_branch_info *branch_info, unsigned int pos, unsigned int lineno, unsigned int outidx, unsigned int jump_pos)
 {
 	vld_set_add(branch_info->ends, pos);
-	branch_info->branches[pos].out[outidx] = jump_pos;
+	if (branch_info->branches[pos].outs_count < VLD_BRANCH_MAX_OUTS) {
+		branch_info->branches[pos].outs[branch_info->branches[pos].outs_count] = jump_pos;
+		branch_info->branches[pos].outs_count++;
+	}
+
 	branch_info->branches[pos].start_lineno = lineno;
 }
 
@@ -66,21 +70,23 @@ void vld_only_leave_first_catch(zend_op_array *opa, vld_branch_info *branch_info
 		position++;
 	}
 
-#if PHP_VERSION_ID >= 70100
-	exit_jmp = position + ((signed int) opa->opcodes[position].extended_value / sizeof(zend_op));
-#else
-	exit_jmp = opa->opcodes[position].extended_value;
-#endif
-
 	if (opa->opcodes[position].opcode != ZEND_CATCH) {
 		return;
 	}
 
-	if (opa->opcodes[exit_jmp].opcode == ZEND_FETCH_CLASS) {
-		exit_jmp++;
-	}
-	if (opa->opcodes[exit_jmp].opcode == ZEND_CATCH) {
-		vld_only_leave_first_catch(opa, branch_info, exit_jmp);
+	if (!opa->opcodes[position].result.num) {
+#if PHP_VERSION_ID >= 70100
+		exit_jmp = position + ((signed int) opa->opcodes[position].extended_value / sizeof(zend_op));
+#else
+		exit_jmp = opa->opcodes[position].extended_value;
+#endif
+
+		if (opa->opcodes[exit_jmp].opcode == ZEND_FETCH_CLASS) {
+			exit_jmp++;
+		}
+		if (opa->opcodes[exit_jmp].opcode == ZEND_CATCH) {
+			vld_only_leave_first_catch(opa, branch_info, exit_jmp);
+		}
 	}
 
 	vld_set_remove(branch_info->entry_points, position);
@@ -106,7 +112,8 @@ void vld_branch_post_process(zend_op_array *opa, vld_branch_info *branch_info)
 	for (i = 0; i < branch_info->starts->size; i++) {
 		if (vld_set_in(branch_info->starts, i)) {
 			if (in_branch) {
-				branch_info->branches[last_start].out[0] = i;
+				branch_info->branches[last_start].outs_count = 1;
+				branch_info->branches[last_start].outs[0] = i;
 				branch_info->branches[last_start].end_op = i-1;
 				branch_info->branches[last_start].end_lineno = branch_info->branches[i].start_lineno;
 			}
@@ -114,8 +121,11 @@ void vld_branch_post_process(zend_op_array *opa, vld_branch_info *branch_info)
 			in_branch = 1;
 		}
 		if (vld_set_in(branch_info->ends, i)) {
-			branch_info->branches[last_start].out[0] = branch_info->branches[i].out[0];
-			branch_info->branches[last_start].out[1] = branch_info->branches[i].out[1];
+			size_t j;
+			for (j = 0; j < branch_info->branches[i].outs_count; j++) {
+				branch_info->branches[last_start].outs[j] = branch_info->branches[i].outs[j];
+			}
+			branch_info->branches[last_start].outs_count = branch_info->branches[i].outs_count;
 			branch_info->branches[last_start].end_op = i;
 			branch_info->branches[last_start].end_lineno = branch_info->branches[i].start_lineno;
 			in_branch = 0;
@@ -185,10 +195,10 @@ static int vld_path_exists(vld_path *path, unsigned int elem1, unsigned int elem
 
 static void vld_branch_find_path(unsigned int nr, vld_branch_info *branch_info, vld_path *prev_path)
 {
-	int out0, out1;
 	unsigned int last;
 	vld_path *new_path;
 	int found = 0;
+	size_t i = 0;
 
 	if (branch_info->paths_count > 255/*65535*/) {
 		return;
@@ -196,18 +206,15 @@ static void vld_branch_find_path(unsigned int nr, vld_branch_info *branch_info,
 
 	new_path = vld_path_new(prev_path);
 	vld_path_add(new_path, nr);
-	out0 = branch_info->branches[nr].out[0];
-	out1 = branch_info->branches[nr].out[1];
 
 	last = vld_branch_find_last_element(new_path);
 
-	if (out0 != 0 && out0 != VLD_JMP_EXIT && !vld_path_exists(new_path, last, out0)) {
-		vld_branch_find_path(out0, branch_info, new_path);
-		found = 1;
-	}
-	if (out1 != 0 && out1 != VLD_JMP_EXIT && !vld_path_exists(new_path, last, out1)) {
-		vld_branch_find_path(out1, branch_info, new_path);
-		found = 1;
+	for (i = 0; i < branch_info->branches[nr].outs_count; i++) {
+		int out = branch_info->branches[nr].outs[i];
+		if (out != 0 && out != VLD_JMP_EXIT && !vld_path_exists(new_path, last, out)) {
+			vld_branch_find_path(out, branch_info, new_path);
+			found = 1;
+		}
 	}
 	if (!found) {
 		vld_branch_info_add_path(branch_info, new_path);
@@ -248,18 +255,13 @@ void vld_branch_info_dump(zend_op_array *opa, vld_branch_info *branch_info TSRML
 				if (vld_set_in(branch_info->entry_points, i)) {
 					fprintf(VLD_G(path_dump_file), "\t%s_ENTRY -> %s_%d\n", fname, fname, i);
 				}
-				if (branch_info->branches[i].out[0]) {
-					if (branch_info->branches[i].out[0] == VLD_JMP_EXIT) {
-						fprintf(VLD_G(path_dump_file), "\t%s_%d -> %s_EXIT;\n", fname, i, fname);
-					} else {
-						fprintf(VLD_G(path_dump_file), "\t%s_%d -> %s_%d;\n", fname, i, fname, branch_info->branches[i].out[0]);
-					}
-				}
-				if (branch_info->branches[i].out[1]) {
-					if (branch_info->branches[i].out[1] == VLD_JMP_EXIT) {
-						fprintf(VLD_G(path_dump_file), "\t%s_%d -> %s_EXIT;\n", fname, i, fname);
-					} else {
-						fprintf(VLD_G(path_dump_file), "\t%s_%d -> %s_%d;\n", fname, i, fname, branch_info->branches[i].out[1]);
+				for (j = 0; j < branch_info->branches[i].outs_count; j++) {
+					if (branch_info->branches[i].outs[j]) {
+						if (branch_info->branches[i].outs[j] == VLD_JMP_EXIT) {
+							fprintf(VLD_G(path_dump_file), "\t%s_%d -> %s_EXIT;\n", fname, i, fname);
+						} else {
+							fprintf(VLD_G(path_dump_file), "\t%s_%d -> %s_%d;\n", fname, i, fname, branch_info->branches[i].outs[j]);
+						}
 					}
 				}
 			}
@@ -276,11 +278,11 @@ void vld_branch_info_dump(zend_op_array *opa, vld_branch_info *branch_info TSRML
 				i,
 				branch_info->branches[i].end_op
 			);
-			if (branch_info->branches[i].out[0]) {
-				printf("; out1: %3d", branch_info->branches[i].out[0]);
-			}
-			if (branch_info->branches[i].out[1]) {
-				printf("; out2: %3d", branch_info->branches[i].out[1]);
+
+			for (j = 0; j < branch_info->branches[i].outs_count; j++) {
+				if (branch_info->branches[i].outs[j]) {
+					printf("; out%d: %3d", j, branch_info->branches[i].outs[j]);
+				}
 			}
 			printf("\n");
 		}
