@@ -1,6 +1,6 @@
 /*
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2013 Derick Rethans                               |
+   | Copyright (c) 1997-2016 Derick Rethans                               |
    +----------------------------------------------------------------------+
    | This source file is subject to the 2-Clause BSD license which is     |
    | available through the LICENSE file, or online at                     |
@@ -69,11 +69,11 @@ int vld_printf(FILE *stream, const char* fmt, ...);
 # define ZVAL_VALUE_STRING_TYPE         zend_string
 # define ZVAL_STRING_VALUE(s)           (s.str)->val
 # define ZVAL_STRING_LEN(s)             (s.str)->len
-# define ZSTRING_VALUE(s)               (s)->val
+# define ZSTRING_VALUE(s)               (s ? (s)->val : NULL)
 # define OPARRAY_VAR_NAME(v)            (v)->val
 #else
 # define ZHASHKEYSTR(k) ((k)->arKey)
-# define ZHASHKEYLEN(k) ((k)->nKeyLength)
+# define ZHASHKEYLEN(k) ((k)->nKeyLength - 1)
 # define PHP_URLENCODE_NEW_LEN(v) , &(v)
 
 # define ZVAL_VALUE_TYPE                zvalue_value
