@@ -58,16 +58,18 @@
 #define OP2_BRK_CONT 1<<10
 #define OP1_CLASS    1<<11
 #define RES_CLASS    1<<12
+#define OP2_JMP_ARRAY    1<<13
 
 #define SPECIAL    0xff
 
 // special op-type flags
-#define VLD_IS_OPLINE 1<<13
-#define VLD_IS_OPNUM  1<<14
-#define VLD_IS_CLASS  1<<15
-#define OP2_INCLUDE   1<<16
-#define EXT_VAL_JMP_REL   1<<17
-#define EXT_VAL_JMP_ABS   1<<18
+#define VLD_IS_OPLINE 1<<20
+#define VLD_IS_OPNUM  1<<21
+#define VLD_IS_CLASS  1<<22
+#define OP2_INCLUDE   1<<23
+#define EXT_VAL_JMP_REL   1<<24
+#define EXT_VAL_JMP_ABS   1<<25
+#define VLD_IS_JMP_ARRAY  1<<26
 
 typedef struct _op_usage {
 	char *name;
